@@ -302,12 +302,12 @@ class Trimesh(Geometry3D):
           Indexes of self.vertices
         """
         if values is None or len(values) == 0:
-            if 'faces' in self._data:
-                del self._data['faces']
-            return
-        values = np.asanyarray(values, dtype=np.int64)
+            return self._data.data.pop('faces', None)
+        if not (isinstance(values, np.ndarray) and values.dtype == np.int64):
+            values = np.asanyarray(values, dtype=np.int64)
+
         # automatically triangulate quad faces
-        if util.is_shape(values, (-1, 4)):
+        if len(values.shape) == 2 and values.shape[1] == 4:
             log.info('triangulating quad faces')
             values = geometry.triangulate_quads(values)
         self._data['faces'] = values
@@ -1050,7 +1050,7 @@ class Trimesh(Geometry3D):
         referenced : (len(self.vertices), ) bool
           Which vertices are referenced by a face
         """
-        referenced = np.zeros(len(self.vertices), dtype=np.bool)
+        referenced = np.zeros(len(self.vertices), dtype=bool)
         referenced[self.faces] = True
         return referenced
 
@@ -1091,20 +1091,19 @@ class Trimesh(Geometry3D):
 
     def merge_vertices(self, **kwargs):
         """
-        If a mesh has vertices that are closer than
-        trimesh.constants.tol.merge reindex faces to reference
-        the same index for both vertices.
+        Removes duplicate vertices, grouped by position and
+        optionally texture coordinate and normal.
 
         Parameters
         -------------
         mesh : Trimesh object
           Mesh to merge vertices on
-        use_tex : bool
-          If True for textured meshes merge vertices
-          with identical positions AND UV coordinates.
-        use_norm : bool
-          If True meshes with vertex normals defined will
-          only have vertices merged with identical normal
+        merge_tex : bool
+          If True textured meshes with UV coordinates will
+          have vertices merged regardless of UV coordinates
+        merge_norm : bool
+          If True, meshes with vertex normals will have
+          vertices merged ignoring different normals
         digits_vertex : None or int
           Number of digits to consider for vertex position
         digits_norm : int
@@ -1735,7 +1734,7 @@ class Trimesh(Geometry3D):
         """
         # if no facets exit early
         if len(self.facets) == 0:
-            return np.array([], dtype=np.bool)
+            return np.array([], dtype=bool)
 
         # facets plane, origin and normal
         normals = self.facets_normal
@@ -1745,7 +1744,7 @@ class Trimesh(Geometry3D):
         convex = self.convex_hull.vertices.view(np.ndarray).copy()
 
         # boolean mask for which facets are on convex hull
-        on_hull = np.zeros(len(self.facets), dtype=np.bool)
+        on_hull = np.zeros(len(self.facets), dtype=bool)
 
         for i, normal, origin in zip(range(len(normals)), normals, origins):
             # a facet plane is on the convex hull if every vertex
@@ -1826,7 +1825,7 @@ class Trimesh(Geometry3D):
                              n_samples=1,
                              threshold=0.0):
         """
-        Computes stable orientations of a mesh and their quasi-static probabilites.
+        Computes stable orientations of a mesh and their quasi-static probabilities.
 
         This method samples the location of the center of mass from a multivariate
         gaussian (mean at com, cov equal to identity times sigma) over n_samples.
@@ -2032,10 +2031,10 @@ class Trimesh(Geometry3D):
 
         Parameters
         ------------
-        plane_normal: (3) vector for plane normal
-          Normal vector of section plane
         plane_origin : (3, ) float
           Point on the cross section plane
+        plane_normal: (3) vector for plane normal
+          Normal vector of section plane
         heights : (n, ) float
           Each section is offset by height along
           the plane normal.
@@ -2167,7 +2166,7 @@ class Trimesh(Geometry3D):
         Remove all vertices in the current mesh which are not
         referenced by a face.
         """
-        referenced = np.zeros(len(self.vertices), dtype=np.bool)
+        referenced = np.zeros(len(self.vertices), dtype=bool)
         referenced[self.faces] = True
 
         inverse = np.zeros(len(self.vertices), dtype=np.int64)
