@@ -90,7 +90,7 @@ def mesh_plane(mesh,
         # of both on- edge cases ([6,16])
         # this is so that for regions that are co-planar with the the section plane
         # we don't end up with an invalid boundary
-        key = np.zeros(29, dtype=np.bool)
+        key = np.zeros(29, dtype=bool)
         key[16] = True
         one_edge = key[coded]
 
@@ -201,10 +201,10 @@ def mesh_multiplane(mesh,
     -------------
     mesh : trimesh.Trimesh
         Geometry to be sliced by planes
-    plane_normal : (3,) float
-        Normal vector of plane
     plane_origin : (3,) float
         Point on a plane
+    plane_normal : (3,) float
+        Normal vector of plane
     heights : (m,) float
       Offset distances from plane to slice at:
       at `height=0` it will be exactly on the passed plane.
@@ -720,8 +720,13 @@ def slice_mesh_plane(mesh,
             if check.astype(np.float64).mean() > 0.5:
                 ff = np.fliplr(ff)
 
+            # check vertices to see which ones are on plane
+            on_plane = np.abs(np.dot(vertices - origin, normal)) < tol.merge
             # add cap vertices and faces and reindex
-            vertices, faces = util.append_faces([vertices, vf], [faces, ff])
+            # remove any original faces which are coplanar with cap plane
+            # as these will be replaced by newly generated faces
+            vertices, faces = util.append_faces(
+                [vertices, vf], [faces[~on_plane[faces].all(axis=1)], ff])
 
             # Update mesh with cap (processing needed to merge vertices)
             sliced_mesh = Trimesh(vertices=vertices, faces=faces)
