@@ -7,7 +7,6 @@ Trimesh, Scene, PointCloud, and Path objects.
 
 Works on all major platforms: Windows, Linux, and OSX.
 """
-import platform
 import collections
 import numpy as np
 
@@ -232,8 +231,14 @@ class SceneViewer(pyglet.window.Window):
         kwargs **
           Passed to rendering.convert_to_vertexlist
         """
-        # convert geometry to constructor args
-        args = rendering.convert_to_vertexlist(geometry, **kwargs)
+        try:
+            # convert geometry to constructor args
+            args = rendering.convert_to_vertexlist(geometry, **kwargs)
+        except BaseException:
+            util.log.warning('failed to add geometry `{}`'.format(name),
+                             exc_info=True)
+            return
+
         # create the indexed vertex list
         self.vertex_list[name] = self.batch.add_indexed(*args)
         # save the hash of the geometry
@@ -702,6 +707,10 @@ class SceneViewer(pyglet.window.Window):
         if self._grid:
             self._grid.draw(mode=gl.GL_LINES)
 
+        # save a reference outside of the loop
+        geometry = self.scene.geometry
+        graph = self.scene.graph
+
         while len(node_names) > 0:
             count += 1
             current_node = node_names.popleft()
@@ -710,10 +719,10 @@ class SceneViewer(pyglet.window.Window):
                 continue
 
             # get the transform from world to geometry and mesh name
-            transform, geometry_name = self.scene.graph.get(current_node)
+            transform, geometry_name = graph.get(current_node)
 
             # if no geometry at this frame continue without rendering
-            if geometry_name is None:
+            if geometry_name is None or geometry_name not in self.vertex_list_mode:
                 continue
 
             # if a geometry is marked as fixed apply the inverse view transform
@@ -725,7 +734,7 @@ class SceneViewer(pyglet.window.Window):
                 transform = np.dot(transform, transform_fix)
 
             # get a reference to the mesh so we can check transparency
-            mesh = self.scene.geometry[geometry_name]
+            mesh = geometry[geometry_name]
             if mesh.is_empty:
                 continue
 
@@ -842,14 +851,21 @@ def render_scene(scene,
                  visible=True,
                  **kwargs):
     """
-    Render a preview of a scene to a PNG.
+    Render a preview of a scene to a PNG. Note that
+    whether this works or not highly variable based on
+    platform and graphics driver.
 
     Parameters
     ------------
     scene : trimesh.Scene
       Geometry to be rendered
     resolution : (2,) int or None
-      Resolution in pixels, or set from scene.camera
+      Resolution in pixels or set from scene.camera
+    visible : bool
+      Show a window during rendering. Note that MANY
+      platforms refuse to render with hidden windows
+      and will likely return a blank image; this is a
+      platform issue and cannot be fixed in Python.
     kwargs : **
       Passed to SceneViewer
 
@@ -858,14 +874,9 @@ def render_scene(scene,
     render : bytes
       Image in PNG format
     """
-    window = SceneViewer(scene,
-                         start_loop=False,
-                         visible=visible,
-                         resolution=resolution,
-                         **kwargs)
-
-    if visible is None:
-        visible = platform.system() != 'Linux'
+    window = SceneViewer(
+        scene, start_loop=False, visible=visible,
+        resolution=resolution, **kwargs)
 
     from ..util import BytesIO
 
